// ==============================================================
// MFDsample - sample MFD to demonstrate MFDlib routines
//
// InfoMFD.cpp
// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include "orbitersdk.h"
#include "MFDlib.h"
#include "MFDsample.h"

// ==============================================================
// Global variables

static int g_InfoMFD;

// ==============================================================
// API interface

DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	static char *name = "MFDlib";
	MFDMODESPEC spec;
	spec.name    = name;
	spec.key     = OAPI_KEY_I;	// activation hotkey
	spec.msgproc = InfoMFD::MsgProc;

	g_InfoMFD = oapiRegisterMFDMode (spec);
}

DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{
	oapiUnregisterMFDMode (g_InfoMFD);
}

// ==============================================================
// Info MFD implementation

InfoMFD::InfoMFD (DWORD w, DWORD h, VESSEL *vessel)
: MFD (w, h, vessel)
{
}

InfoMFD::~InfoMFD ()
{
}

// message parser
int InfoMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		return (int)(new InfoMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
	}
	return 0;
}

void InfoMFD::ShowColors(const HDC hDC, const int y, const char *name, const COLORREF c1, const COLORREF c2, const COLORREF c3)
{
	TextXY(hDC, 0, y, WHITE, name);
	TextXY(hDC, 10, y, c1, "LIGHT");
	TextXY(hDC, 16, y, c2, "NORMAL");
	TextXY(hDC, 23, y, c3, "BRIGHT");
	// NB: black is transparent if Transparent MFD set in Launchpad, and NOT using panels
	TextXY(hDC, 30, y, BLACK, c3, "REVERSE");
}

// Orbiter callback routine for screen updates
void InfoMFD::Update (const HDC hDC)
{
	MFDUpdate(hDC, cw, ch);		// tell MFDlib what the character width & height are
	Title(hDC, "MFDlib Sample");

	TextXY(hDC, 20, 0, "FontSize %dx%d", cw, ch);

	// use the MFD variable pV (pointer to VESSEL) to get info on active ship
	TextXY(hDC, 0, 1, YELLOW, "%s called %s", pV->GetClassName(), pV->GetName());

	ShowColors(hDC, 2, "RED", LIGHTRED, RED, BRIGHTRED);
	ShowColors(hDC, 3, "GREEN", LIGHTGREEN, GREEN, BRIGHTGREEN);
	ShowColors(hDC, 4, "YELLOW", LIGHTYELLOW, YELLOW, BRIGHTYELLOW);
	ShowColors(hDC, 5, "BLUE", LIGHTBLUE, BLUE, BRIGHTBLUE);
	ShowColors(hDC, 6, "MAGENTA", LIGHTMAGENTA, MAGENTA, BRIGHTMAGENTA);
	ShowColors(hDC, 7, "CYAN", LIGHTCYAN, CYAN, BRIGHTCYAN);
	ShowColors(hDC, 8, "WHITE", LIGHTWHITE, WHITE, BRIGHTWHITE);

	SetMFDColor(hDC, WHITE);	// reset back to "normal" colors
	TextXY(hDC, 0, 10, "          1         2         3         4");
	TextXY(hDC, 0, 11, "01234567890123456789012345678901234567890");

	for (int i=12; i<30; i++)
		TextXY(hDC, i, i, "+ (%d, %d)", i, i);

	TextXY(hDC, 0, 18, "Dock Ports = %d", pV->DockCount());
	TextXY(hDC, 0, 19, "Mass = %.1f", pV->GetEmptyMass());
	TextXY(hDC, 0, 20, "Engine Level = %03.0f%%", pV->GetEngineLevel(ENGINE_MAIN)*100.0);
}
