#include "ADI.h"
#include <gl/gl.h>
#include <gl/glu.h>
#include "utils/FloatUtils.h"

ADI::ADI(int x, int y, int width, int height) {
	this->x = x; this->y = y; this->width = width; this->height = height;
	for (int i=0; i < 8; i++)
		NSEW[i] = 0.0;
	for (int i=0; i < 2; i++) {
		vVector[i] = 0.0;
		t10Vector[i] = 0.0;
	}
	GLuint      PixelFormat;  
	BITMAPINFOHEADER BIH;
	int iSize=sizeof(BITMAPINFOHEADER);
	BIH.biSize=iSize;
	BIH.biWidth=width;
	BIH.biHeight=height;
	BIH.biPlanes=1;
	BIH.biBitCount=16;//default is 16.
	BIH.biCompression=BI_RGB;
	BIH.biSizeImage=0;
	void* m_pBits;
	hDC=CreateCompatibleDC(NULL);//we make a new DC and DIbitmap for OpenGL to draw onto
	static  PIXELFORMATDESCRIPTOR pfd2;
	DescribePixelFormat(hDC,1,sizeof(PIXELFORMATDESCRIPTOR),&pfd2);//just get a random pixel format.. 
	BIH.biBitCount=pfd2.cColorBits;//to get the current bit depth.. !?
	hBMP=CreateDIBSection(hDC,(BITMAPINFO*)&BIH,DIB_RGB_COLORS,&m_pBits,NULL,0);
	hBMP_old=(HBITMAP)SelectObject(hDC,hBMP);
	static  PIXELFORMATDESCRIPTOR pfd={                             // pfd Tells Windows How We Want Things To Be
        sizeof(PIXELFORMATDESCRIPTOR),                              // Size Of This Pixel Format Descriptor
        1,                                                          // Version Number
		PFD_DRAW_TO_BITMAP |                                        // Format Must Support Bitmap Rendering
        PFD_SUPPORT_OPENGL |									
		PFD_SUPPORT_GDI,											// Format Must Support OpenGL,                                           
		0,//        PFD_TYPE_RGBA,                                              // Request An RGBA Format
        16,															// Select Our Color Depth
        0, 0, 0, 0, 0, 0,                                           // Color Bits Ignored
        0,//1,                                                          // No Alpha Buffer
        0,                                                          // Shift Bit Ignored
        0,                                                          // No Accumulation Buffer
        0, 0, 0, 0,                                                 // Accumulation Bits Ignored
        0,//16,                                                         // 16Bit Z-Buffer (Depth Buffer)  
        0,                                                          // No Stencil Buffer
        0,                                                          // No Auxiliary Buffer
        0,//PFD_MAIN_PLANE,                                             // Main Drawing Layer
        0,                                                          // Reserved
        0, 0, 0                                                     // Layer Masks Ignored
    };
	pfd.cColorBits=pfd2.cColorBits;//same color depth needed.
	DWORD code;
	code=GetLastError();
	PixelFormat=ChoosePixelFormat(hDC,&pfd);// now pretend we want a new format
	int ret;
	ret=SetPixelFormat(hDC,PixelFormat,&pfd);
	code=GetLastError();
	hRC=wglCreateContext(hDC);
	ret=wglMakeCurrent(hDC,hRC);					//all standard OpenGL init so far

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_POLYGON_OFFSET_FILL);
	glPolygonOffset (1.0, 2);

	glViewport(0, 0, width, height);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);           // Panel Background color

	CreateDisplayLists();
}

// Destructor
ADI::~ADI() {
	wglMakeCurrent(NULL,NULL);	//standard OpenGL release
	wglDeleteContext(hRC);
	hRC=NULL;
	SelectObject(hDC,hBMP_old);//remember to delete DC and bitmap memory we created
	DeleteObject(hBMP);
	DeleteDC(hDC);
}


void ADI::CreateDisplayLists() {
	displayLists[0] = glGenLists(NUM_DLS);

	for (int i = 1; i < NUM_DLS; i++)
		displayLists[i] = displayLists[0] + i;

	float x, y, z, scale;
	int elev, azi, elev2;

	glNewList (displayLists[DL_HEMISPHERE], GL_COMPILE);
		glBegin (GL_TRIANGLE_FAN);
			glVertex3f (0.0f, 1.0f, 0.0f); // top vertex
			elev = 80;
			y = sin (elev*RADf);
			for (azi = 360; azi >= 0; azi -= 10) { // run in a circle at +80 degrees
				x = cos (elev*RADf) * cos (azi*RADf);
				z = cos (elev*RADf) * sin (azi*RADf);
				glVertex3f (x, y, z);
			}
		glEnd();

		for (elev = 70; elev >= 0; elev -= 10) {
			elev2 = elev + 10;
			azi = 0;
			glBegin (GL_QUAD_STRIP);

				y = sin (elev*RADf);
				x = cos (elev*RADf) * cos (azi*RADf);
				z = cos (elev*RADf) * sin (azi*RADf);
				glVertex3f (x, y, z);

				y = sin (elev2*RADf);
				x = cos (elev2*RADf) * cos (azi*RADf);
				z = cos (elev2*RADf) * sin (azi*RADf);
				glVertex3f (x, y, z);

				for (azi = 10; azi <= 360; azi += 10) {
					y = sin (elev*RADf);
					x = cos (elev*RADf) * cos (azi*RADf);
					z = cos (elev*RADf) * sin (azi*RADf);
					glVertex3f (x, y, z);

					y = sin (elev2*RADf);
					x = cos (elev2*RADf) * cos (azi*RADf);
					z = cos (elev2*RADf) * sin (azi*RADf);
					glVertex3f (x, y, z);
				}
			glEnd ();
		}
	glEndList();

	glNewList (displayLists[DL_CIRCLE_XZ], GL_COMPILE);
		glBegin (GL_LINE_LOOP);
			for (azi = 0; azi < 360; azi += 10) {
				x = sin (azi*RADf);
				z = cos (azi*RADf);
				glVertex3f (x, 0.0f, z);
			}
		glEnd();
	glEndList();

	glNewList (displayLists[DL_CIRCLE_XY], GL_COMPILE);
		glBegin (GL_LINE_LOOP);
			for (azi = 0; azi < 360; azi += 10) {
				x = cos (azi*RADf);
				y = sin (azi*RADf);
				glVertex3f (x, y, 0.0f);
			}
		glEnd();
	glEndList();

	glNewList (displayLists[DL_LATITUDE_MAJOR], GL_COMPILE);
		for (int elev = 30; elev < 90; elev += 30) {
			glPushMatrix();
				glTranslatef(0.0f, sin (elev*RADf), 0.0f);
				scale = cos (elev*RADf);
				glScalef (scale, scale, scale);
				glCallList(displayLists[DL_CIRCLE_XZ]);
			glPopMatrix();
		}
	glEndList();

	glNewList (displayLists[DL_LATITUDE_MINOR], GL_COMPILE);
		for (int elev = 10; elev < 90; elev += 10) {
			if (elev%30 != 0) {
				glPushMatrix();
					glTranslatef(0.0f, sin (elev*RADf), 0.0f);
					scale = cos (elev*RADf);
					glScalef (scale, scale, scale);
					glCallList(displayLists[DL_CIRCLE_XZ]);
				glPopMatrix();
			}
		}
	glEndList();

	glNewList (displayLists[DL_LONGITUDE], GL_COMPILE);
		for (int azi = 0; azi < 360; azi += 30) {
			glPushMatrix();
				glRotatef ((float)azi, 0.0f, 1.0f, 0.0f);
				glCallList(displayLists[DL_CIRCLE_XY]);
			glPopMatrix();
		}
	glEndList();

	glNewList (displayLists[DL_BALL], GL_COMPILE);
		glPushMatrix();
			glColor3f (0.0f, 0.63f, 0.65f);				// sky background colour
			glCallList (displayLists[DL_HEMISPHERE]);	//render

			glColor3f (0.6f, 0.6f, 0.97f);				// TODO: sky line colour
			glCallList (displayLists[DL_LATITUDE_MAJOR]);

			glColor3f (0.0f, 0.97f, 0.03f); // horizon line colour
			glCallList (displayLists[DL_CIRCLE_XZ]); //render

			glColor3f (0.5f, 0.5f, 0.5f); // TODO: longitude colour
			glCallList (displayLists[DL_LONGITUDE]);

			glRotatef(180.0f, 0.0f, 0.0f, 1.0f);
			glColor3f (0.38f, 0.25f, 0.15f); // earth background colour
			glCallList (displayLists[DL_HEMISPHERE]);	//render

			glColor3f (1.0f, 0.5f, 0.28f);
			glCallList (displayLists[DL_LATITUDE_MAJOR]);
		glPopMatrix();
	glEndList();
}

void ADI::Draw(HDC hDC, float pitch, float roll, float yaw, float alpha, float beta, float pitchrate, float yawrate, float zoom) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);             // Clear The Screen And The Depth Buffer        

	double zoomd = 1.0 / zoom;
	double aspect = (double)width / (double)height;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if (aspect > 1)  // wider than tall
		glOrtho (-zoomd * aspect, zoomd * aspect, -zoomd, zoomd, 2.0, 4.0);
	else
		glOrtho (-zoomd, zoomd, -zoomd / aspect, zoomd / aspect, 2.0, 4.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glTranslatef(0.0f, 0.0f, -3.0f);
	glRotatef (roll, 0.0f, 0.0f, 1.0f); // roll
	glRotatef (pitch, 1.0f, 0.0f, 0.0f); // pitch
	glRotatef (yaw, 0.0f, 1.0f, 0.0f); // yaw

	glCallList (displayLists[DL_BALL]);

	glEnable(GL_LINE_STIPPLE);
	glLineStipple(5, 0x5555);

	glColor3f (0.6f, 0.6f, 0.97f);				// TODO: sky line colour
	glCallList (displayLists[DL_LATITUDE_MINOR]);

	glPushMatrix();
		glColor3f (1.0f, 0.5f, 0.28f);
		glScalef (-1.0f, -1.0f, -1.0f);
		glCallList (displayLists[DL_LATITUDE_MINOR]);
	glPopMatrix();

	glDisable(GL_LINE_STIPPLE);

	glFlush();
	glFinish();
	BitBlt (hDC, x, y, width, height, this->hDC, 0, 0, SRCCOPY);

	CalcCompass (pitch, roll, yaw, alpha, beta, pitchrate, yawrate);
}

void ADI::CalcCompass(float pitch, float roll, float yaw, float alpha, float beta, float pitchrate, float yawrate) {
	GLdouble model[16];
	GLdouble proj[16];
	GLint view[4];
	GLdouble z;

	GLdouble fp[3];
	float fpPitch, fpHeading;

	glGetDoublev(GL_MODELVIEW_MATRIX, model);
	glGetDoublev(GL_PROJECTION_MATRIX, proj);
	glGetIntegerv(GL_VIEWPORT, view);
	gluProject ( 0.0, 0.0,  1.0, model, proj, view,  NSEW+0, NSEW+1, &z);
	gluProject ( 0.0, 0.0, -1.0, model, proj, view,  NSEW+2, NSEW+3, &z);
	gluProject ( 1.0, 0.0,  0.0,  model, proj, view, NSEW+4, NSEW+5, &z);
	gluProject (-1.0, 0.0,  0.0,  model, proj, view, NSEW+6, NSEW+7, &z);


	for (int i=0; i < 8; i+=2)
		CheckRange (NSEW[i], (double)0, (double)width);
	for (int i=1; i < 8; i+=2) {
		CheckRange (NSEW[i], (double)0, (double)height);
		NSEW[i] = height - NSEW[i]; // invert y coords
	}

	fpPitch = pitch - (cos(roll*RADf)*alpha) - (sin(roll*RADf)*beta);
	fpHeading = -yaw - (sin(roll*RADf)*alpha) - (cos(roll*RADf)*beta);

	fp[0] = cos (fpPitch*RADf) * sin(fpHeading*RADf);
	fp[1] = sin (fpPitch*RADf);
	fp[2] = cos (fpPitch*RADf) * cos(fpHeading*RADf);

	gluProject (fp[0], fp[1], fp[2], model, proj, view, vVector, vVector+1, &z);

	CheckRange (vVector[0], (double)0, (double)width);
	CheckRange (vVector[1], (double)0, (double)height);
	vVector[1] = height - vVector[1]; // invert y coords

	fpPitch = pitch + pitchrate*3;
	fpHeading = -yaw + yawrate*3;

	fp[0] = cos (fpPitch*RADf) * sin(fpHeading*RADf);
	fp[1] = sin (fpPitch*RADf);
	fp[2] = cos (fpPitch*RADf) * cos(fpHeading*RADf);

	gluProject (fp[0], fp[1], fp[2], model, proj, view, t10Vector, t10Vector+1, &z);

	CheckRange (t10Vector[0], (double)0, (double)width);
	CheckRange (t10Vector[1], (double)0, (double)height);
	t10Vector[1] = height - t10Vector[1]; // invert y coords

}
